package ink.wgink.interfaces.menu;

import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.dtos.menu.MenuDTO;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IMenuBaseService
 * @Description: 菜单接口
 * @Author: wanggeng
 * @Date: 2021/2/10 9:17 下午
 * @Version: 1.0
 */
public interface IMenuBaseService {

    /**
     * 统一用户菜单
     */
    String MENU_UNIFIED_USER = "unified-user";

    /**
     * 菜单详情
     *
     * @param params
     * @return
     */
    MenuDTO get(Map<String, Object> params);

    /**
     * 菜单列表
     *
     * @param params
     * @return
     */
    List<MenuDTO> list(Map<String, Object> params);

    /**
     * 菜单列表
     *
     * @param menuParentId 上级ID
     * @return
     */
    List<MenuDTO> listByParentId(String menuParentId);

    /**
     * 菜单列表
     *
     * @param menuIds 菜单ID列表
     * @return
     */
    List<MenuDTO> listByIds(List<String> menuIds);

    /**
     * 菜单列表
     *
     * @param menuParentId 上级菜单ID
     * @param menuIds      菜单ID列表
     * @return
     */
    List<MenuDTO> listByParentIdAndIds(String menuParentId, List<String> menuIds);

    /**
     * 菜单列表，递归获取全部内容
     *
     * @param params
     * @return
     */
    List<MenuDTO> listAll(Map<String, Object> params);

    /**
     * 菜单列表，递归获取全部内容
     *
     * @param menuParentId 上级菜单ID
     * @return
     */
    List<MenuDTO> listAllByParentId(String menuParentId);

    /**
     * 菜单列表，递归获取全部内容
     *
     * @param menuParentId 上级菜单ID
     * @param menuIds      菜单ID列表
     * @return
     */
    List<MenuDTO> listAllByParentIdAndIds(String menuParentId, List<String> menuIds);

    /**
     * zTree列表
     *
     * @param params
     * @return
     */
    List<ZTreeDTO> listZTree(Map<String, Object> params);

    /**
     * easyui菜单列表
     *
     * @param page
     * @return
     */
    SuccessResultList<List<MenuDTO>> listPage(ListPage page);

    /**
     * 通过用户获取菜单
     *
     * @param params
     * @return
     */
    List<String> listMenuIdByUser(Map<String, Object> params);

}
