package ink.wgink.login.oauth2.client.remote.department;

import ink.wgink.annotation.rpc.rest.RemoteService;
import ink.wgink.annotation.rpc.rest.method.RemoteGetMethod;
import ink.wgink.annotation.rpc.rest.method.RemotePostMethod;
import ink.wgink.annotation.rpc.rest.params.*;
import ink.wgink.pojo.dtos.department.DepartmentUserDTO;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.pojo.vos.IdsVO;
import org.springframework.web.bind.annotation.PathVariable;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IDepartmentUserRemoteService
 * @Description: 组织部门用户调用
 * @Author: wanggeng
 * @Date: 2021/9/22 10:42 下午
 * @Version: 1.0
 */
@RemoteService("/resource/department/user")
public interface IDepartmentUserRemoteService {

    @RemoteGetMethod("/list")
    List<DepartmentUserDTO> list(@RemoteServerParams String userCenter, @RemotePathParams("access_token") String accessToken, @RemoteQueryParamsMap Map<String, Object> params);

    @RemoteGetMethod("/list/{department}")
    List<DepartmentUserDTO> listByDepartmentId(@RemoteServerParams String userCenter, @RemotePathParams("departmentId") String departmentId, @RemotePathParams("access_token") String accessToken);

    @RemotePostMethod("/list/department-ids")
    List<DepartmentUserDTO> listByDepartmentIds(@RemoteServerParams String userCenter, @RemotePathParams("access_token") String accessToken, @RemoteJsonBodyParams IdsVO idsVO);

    @RemoteGetMethod("/list-user-id/{departmentId}")
    List<String> listUserId(@RemoteServerParams String userCenter, @RemotePathParams("departmentId") String departmentId, @RemotePathParams("access_token") String accessToken);

    @RemotePostMethod("/list-user-id/department-ids")
    List<String> listUserIdByDepartmentIds(@RemoteServerParams String userCenter, @RemotePathParams("access_token") String accessToken, @RemoteJsonBodyParams IdsVO idsVO);

    @RemotePostMethod("/list-user-id/department-id/{departmentId}/user-ids")
    List<String> listUserIdByDepartmentIdAndUserIds(@RemoteServerParams String userCenter, @RemotePathParams("departmentId") String departmentId, @RemotePathParams("access_token") String accessToken, @RemoteJsonBodyParams IdsVO idsVO);

    @RemotePostMethod("/list-user-id/department-ids/user-ids")
    List<String> listUserIdByDepartmentIdsAndUserIds(@RemoteServerParams String userCenter, @RemotePathParams("access_token") String accessToken, @RemoteJsonBodyParams IdsVO idsVO);

    @RemoteGetMethod("/listpage/{departmentId}")
    SuccessResultList<List<DepartmentUserDTO>> listPage(@RemoteServerParams String userCenter, @PathVariable("departmentId") String departmentId, @RemotePathParams("access_token") String accessToken, @RemoteQueryParams("page") int page, @RemoteQueryParams("rows") int rows, @RemoteQueryParamsMap Map<String, Object> params);

}
