/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ink.wgink.module.activiti.controller.api;

import com.alibaba.fastjson.JSONObject;
import com.fasterxml.jackson.databind.ObjectMapper;
import ink.wgink.exceptions.base.SystemException;
import ink.wgink.pojo.result.ErrorResult;
import io.swagger.annotations.*;
import org.activiti.editor.constants.ModelDataJsonConstants;
import org.activiti.engine.RepositoryService;
import org.activiti.engine.repository.Model;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * 用于在线编辑器编辑工作流模型
 *
 * @author Tijs Rademakers
 */
@RestController
@RequestMapping("service")
public class ActivitiModelEditorController implements ModelDataJsonConstants {

    protected static final Logger LOG = LoggerFactory.getLogger(ActivitiModelEditorController.class);

    @Autowired
    private RepositoryService repositoryService;

    @Autowired
    private ObjectMapper objectMapper;

    @ApiOperation(value = "模型编辑时的json数据", notes = "模型编辑时的json数据接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "modelId", value = "模型ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("model/{modelId}/json")
    public JSONObject getEditorJson(@PathVariable String modelId) {
        JSONObject result = null;
        Model model = repositoryService.getModel(modelId);

        if (model != null) {
            try {
                if (StringUtils.isNotEmpty(model.getMetaInfo())) {
                    result = JSONObject.parseObject(model.getMetaInfo());
                } else {
                    result = new JSONObject();
                    result.put(MODEL_NAME, model.getName());
                }
                result.put(MODEL_ID, model.getId());
                String readTree = new String(repositoryService.getModelEditorSource(model.getId()), "utf-8");
                JSONObject editorJson = JSONObject.parseObject(readTree);
                result.put("model", editorJson);
            } catch (Exception e) {
                LOG.error(e.getMessage(), e);
                throw new SystemException(e.getMessage(), e);
            }
        }
        return result;
    }

}
