/* Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package ink.wgink.module.activiti.controller.staticfile;

import ink.wgink.util.ResourceUtil;
import ink.wgink.util.request.StaticResourceRequestUtil;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;

/**
 * diagram静态资源
 *
 * @author Tijs Rademakers
 */
@RestController
@RequestMapping("static/diagram-viewer")
public class ActivitiDiagramViewerStaticController {

    @GetMapping("{fileName}")
    public void root(HttpServletResponse httpServletResponse, @PathVariable("fileName") String fileName) throws IOException {
        InputStream inputStream = ResourceUtil.getJarResourceInputStream("static/diagram-viewer/" + fileName);
        StaticResourceRequestUtil.download(httpServletResponse, inputStream, fileName);
    }

    @GetMapping("images/{fileName}")
    public void viewerImages(HttpServletResponse httpServletResponse, @PathVariable("fileName") String fileName) throws IOException {
        InputStream inputStream = ResourceUtil.getJarResourceInputStream("static/diagram-viewer/images/" + fileName);
        StaticResourceRequestUtil.download(httpServletResponse, inputStream, fileName);
    }

    @GetMapping("images/deployer/{fileName}")
    public void imagesDeployer(HttpServletResponse httpServletResponse, @PathVariable("fileName") String fileName) throws IOException {
        InputStream inputStream = ResourceUtil.getJarResourceInputStream("static/diagram-viewer/images/deployer/" + fileName);
        StaticResourceRequestUtil.download(httpServletResponse, inputStream, fileName);
    }

    @GetMapping("images/deployer/blue/{fileName}")
    public void imagesDeployerBlue(HttpServletResponse httpServletResponse, @PathVariable("fileName") String fileName) throws IOException {
        InputStream inputStream = ResourceUtil.getJarResourceInputStream("static/diagram-viewer/images/deployer/blue/" + fileName);
        StaticResourceRequestUtil.download(httpServletResponse, inputStream, fileName);
    }

    @GetMapping("js/{fileName}")
    public void js(HttpServletResponse httpServletResponse, @PathVariable("fileName") String fileName) throws IOException {
        InputStream inputStream = ResourceUtil.getJarResourceInputStream("static/diagram-viewer/js/" + fileName);
        StaticResourceRequestUtil.download(httpServletResponse, inputStream, fileName);
    }

    @GetMapping("js/jquery/{fileName}")
    public void jsJquery(HttpServletResponse httpServletResponse, @PathVariable("fileName") String fileName) throws IOException {
        InputStream inputStream = ResourceUtil.getJarResourceInputStream("static/diagram-viewer/js/jquery/" + fileName);
        StaticResourceRequestUtil.download(httpServletResponse, inputStream, fileName);
    }

}
