package ink.wgink.module.article.controller.api.category;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.module.article.pojo.dtos.category.CategoryDTO;
import ink.wgink.module.article.pojo.vos.category.CategoryVO;
import ink.wgink.module.article.service.category.ICategoryService;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: ArticleCategoryController
 * @Description: 文章类别
 * @Author: WenG
 * @Date: 2020-04-03 15:20
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "文章类别接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/category")
public class CategoryController extends DefaultBaseController {

    @Autowired
    private ICategoryService categoryService;

    @ApiOperation(value = "新增文章类别", notes = "新增文章类别接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult saveArticleCategory(@RequestBody CategoryVO categoryVO) {
        categoryService.save(categoryVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "删除文章类别(id列表)", notes = "删除文章类别(id列表)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@PathVariable("ids") String ids) throws RemoveException {
        categoryService.remove(Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改文章类别", notes = "修改文章类别接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "categoryId", value = "文章类别ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("update/{categoryId}")
    @CheckRequestBodyAnnotation
    public SuccessResult update(@PathVariable("categoryId") String categoryId, @RequestBody CategoryVO categoryVO) {
        categoryService.update(categoryId, categoryVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "文章类别详情(通过ID)", notes = "文章类别详情(通过ID)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "categoryId", value = "文章类别ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{categoryId}")
    public CategoryDTO getArticleCategoryById(@PathVariable("categoryId") String categoryId) {
        return categoryService.get(categoryId);
    }

    @ApiOperation(value = "文章类别列表", notes = "文章类别列表接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<CategoryDTO> list() throws SearchException {
        Map<String, Object> params = requestParams();
        return categoryService.list(params);
    }

    @ApiOperation(value = "文章类别分页列表", notes = "文章类别分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<CategoryDTO>> listPage(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return categoryService.listPage(page);
    }

}