package ink.wgink.module.article.controller.app.api.category;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.SearchException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.article.pojo.dtos.category.CategoryDTO;
import ink.wgink.module.article.service.category.ICategoryService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: ArticleCategoryController
 * @Description: 文章类别
 * @Author: WenG
 * @Date: 2020-04-03 15:20
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_APP_PREFIX + "文章类别接口")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/category" + ISystemConstant.APP_RELEASE_SUFFIX)
public class CategoryAppController extends DefaultBaseController {

    @Autowired
    private ICategoryService categoryService;

    @ApiOperation(value = "文章类别详情(通过ID)", notes = "文章类别详情(通过ID)接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "categoryId", value = "文章类别ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{categoryId}")
    public CategoryDTO getArticleCategoryById(@PathVariable("categoryId") String categoryId) {
        return categoryService.get(categoryId);
    }

    @ApiOperation(value = "文章类别列表", notes = "文章类别列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<CategoryDTO> list() throws SearchException {
        Map<String, Object> params = requestParams();
        return categoryService.list(params);
    }

    @ApiOperation(value = "文章类别分页列表", notes = "文章类别分页列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<CategoryDTO>> listPage(ListPage page) throws SearchException {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return categoryService.listPage(page);
    }

}