package ink.wgink.module.dictionary.controller.app.api;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.dictionary.pojo.dtos.AreaDTO;
import ink.wgink.module.dictionary.service.IAreaService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.dtos.ZTreeDTO;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: AreaAreaAppController
 * @Description: 地区字典
 * @Author: WangGeng
 * @Date: 2019/11/18 14:00
 * @Version: 1.0
 **/
@Api(tags = ISystemConstant.API_TAGS_APP_PREFIX + "地区字典管理接口")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/area")
public class AreaAppController extends DefaultBaseController {

    @Autowired
    private IAreaService areaService;

    @ApiOperation(value = "地区字典详情（ID查询）", notes = "地区字典详情（ID查询）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "areaId", value = "地区字典ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{areaId}")
    public AreaDTO get(@RequestHeader("token") String token, @PathVariable("areaId") String areaId) {
        return getRelease(areaId);
    }

    @ApiOperation(value = ISystemConstant.API_TAGS_RELEASE_PREFIX + "地区字典详情（ID查询）", notes = ISystemConstant.API_TAGS_RELEASE_PREFIX + "地区字典详情（ID查询）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "areaId", value = "地区字典ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get" + ISystemConstant.RELEASE_SUFFIX + "/{areaId}")
    public AreaDTO getRelease(@PathVariable("areaId") String areaId) {
        return areaService.get(areaId);
    }

    @ApiOperation(value = "地区字典列表（上级ID查询）", notes = "地区字典列表（上级ID查询）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "areaParentId", value = "地区字典上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listbyparentid/{areaParentId}")
    public List<AreaDTO> listByParentId(@RequestHeader("token") String token, @PathVariable("areaParentId") String areaParentId) {
        return listByParentIdRelease(areaParentId);
    }

    @ApiOperation(value = ISystemConstant.API_TAGS_RELEASE_PREFIX + "地区字典列表（上级ID查询）", notes = ISystemConstant.API_TAGS_RELEASE_PREFIX + "地区字典列表（上级ID查询）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "areaParentId", value = "地区字典上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listbyparentid" + ISystemConstant.RELEASE_SUFFIX + "/{areaParentId}")
    public List<AreaDTO> listByParentIdRelease(@PathVariable("areaParentId") String areaParentId) {
        return areaService.listByParentId(areaParentId);
    }

    @ApiOperation(value = "地区字典全部列表（上级ID查询）", notes = "地区字典全部列表（上级ID查询）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "areaParentId", value = "地区字典上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listallbyparentid/{areaParentId}")
    public List<AreaDTO> list(@RequestHeader("token") String token, @PathVariable("areaParentId") String areaParentId) {
        return listAllByParentIdRelease(areaParentId);
    }

    @ApiOperation(value = ISystemConstant.API_TAGS_RELEASE_PREFIX + "地区字典全部列表（上级ID查询）", notes = ISystemConstant.API_TAGS_RELEASE_PREFIX + "地区字典全部列表（上级ID查询）接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "areaParentId", value = "地区字典上级ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listallbyparentid" + ISystemConstant.RELEASE_SUFFIX + "/{areaParentId}")
    public List<AreaDTO> listAllByParentIdRelease(@PathVariable("areaParentId") String areaParentId) {
        return areaService.listAllByParentId(areaParentId);
    }

    @ApiOperation(value = "分页地区字典列表", notes = "分页地区字典列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "parentId", value = "上级ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<AreaDTO>> listPage(@RequestHeader("token") String token, ListPage page) {
        return listPageRelease(page);
    }

    @ApiOperation(value = ISystemConstant.API_TAGS_RELEASE_PREFIX + "分页地区字典列表", notes = ISystemConstant.API_TAGS_RELEASE_PREFIX + "分页地区字典列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "parentId", value = "上级ID", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
            @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
            @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
            @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage" + ISystemConstant.RELEASE_SUFFIX)
    public SuccessResultList<List<AreaDTO>> listPageRelease(ListPage page) {
        Map<String, Object> params = requestParams();
        String areaParentId = "0";
        if (!StringUtils.isBlank(params.get(ISystemConstant.PARAMS_PARENT_ID) == null ? null : params.get(ISystemConstant.PARAMS_PARENT_ID).toString())) {
            areaParentId = params.get(ISystemConstant.PARAMS_PARENT_ID).toString();
        }
        params.put("areaParentId", areaParentId);
        page.setParams(params);
        return areaService.listPage(page);
    }

    @ApiOperation(value = "zTree列表", notes = "zTree列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
            @ApiImplicitParam(name = "id", value = "父ID", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listztree")
    public List<ZTreeDTO> listZTree(@RequestHeader("token") String token) {
        return listZTreeRelease();
    }

    @ApiOperation(value = ISystemConstant.API_TAGS_RELEASE_PREFIX + "zTree列表", notes = ISystemConstant.API_TAGS_RELEASE_PREFIX + "zTree列表接口")
    @ApiImplicitParams({
            @ApiImplicitParam(name = "id", value = "父ID", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listztree" + ISystemConstant.RELEASE_SUFFIX)
    public List<ZTreeDTO> listZTreeRelease() {
        Map<String, Object> params = requestParams();
        String areaParentId = "0";
        if (!StringUtils.isBlank(params.get(ISystemConstant.PARAMS_ID) == null ? null : params.get(ISystemConstant.PARAMS_ID).toString())) {
            areaParentId = params.get(ISystemConstant.PARAMS_ID).toString();
        }
        params.put("areaParentId", areaParentId);
        return areaService.listZTree(params);
    }

}
