package ink.wgink.module.map.service.grid;

import ink.wgink.exceptions.SearchException;
import ink.wgink.module.map.pojo.dto.grid.GridDTO;
import ink.wgink.module.map.pojo.vo.grid.GridVO;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.SuccessResultList;

import java.util.List;
import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IGridService
 * @Description: 网格业务
 * @Author: WangGeng
 * @Date: 2020/10/21 11:14
 * @Version: 1.0
 **/
public interface IGridService {

    /**
     * 保存网格
     *
     * @param gridVO
     * @return
     * @throws Exception
     */
    void save(GridVO gridVO) throws Exception;

    /**
     * 保存网格并返回ID
     *
     * @param gridVO
     * @return
     * @throws Exception
     */
    String saveAndReturnId(GridVO gridVO) throws Exception;


    /**
     * 删除网格
     *
     * @param relationId 关联关系
     */
    void deleteByRelationId(String relationId);

    /**
     * 删除网格
     *
     * @param gridIds 网格ID列表
     */
    void deleteByGridIds(List<String> gridIds);

    /**
     * 删除空网格（没有节点的网格）
     *
     * @param gridIds
     */
    void deleteEmpty(List<String> gridIds);

    /**
     * 更新网格
     *
     * @param gridId 网格ID
     * @param gridVO 更新内容
     * @throws Exception
     */
    void update(String gridId, GridVO gridVO) throws Exception;

    /**
     * 网格列表
     *
     * @param params
     * @return
     */
    List<GridDTO> list(Map<String, Object> params);

    /**
     * 网格列表
     *
     * @param gridIds 网格ID
     * @return
     */
    List<GridDTO> list(List<String> gridIds);

    /**
     * 获取网格列表
     *
     * @param relationId 关联ID
     * @return
     */
    List<GridDTO> listByRelationId(String relationId);

    /**
     * 获取网格列表
     *
     * @param relationIds 关联ID列表
     * @return
     */
    List<GridDTO> listGroupByRelationIds(List<String> relationIds);

    /**
     * 获取全部网格列表
     *
     * @param relationIds
     * @return
     * @throws SearchException
     */
    List<GridDTO> listByRelationIds(List<String> relationIds);

    /**
     * 获取网格分页列表
     *
     * @param relationIds
     * @return
     */
    SuccessResultList<List<GridDTO>> listPageByRelationIds(ListPage page, List<String> relationIds);


}
