package ink.wgink.module.wechat.service.pay.v3.certificates.impl;

import ink.wgink.exceptions.base.SystemException;
import ink.wgink.module.wechat.enums.PayAuthorizationTypeEnum;
import ink.wgink.module.wechat.pojo.pay.v3.certificates.Certificate;
import ink.wgink.module.wechat.request.pay.v3.AbstractPayRequest;
import ink.wgink.module.wechat.request.pay.v3.certificates.CertificatePayRequestImpl;
import ink.wgink.module.wechat.service.pay.BasePayService;
import ink.wgink.module.wechat.service.pay.v3.certificates.ICertificateService;
import ink.wgink.util.date.DateUtil;
import org.springframework.stereotype.Service;
import org.springframework.web.bind.annotation.RequestMethod;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.List;

/**
 * @ClassName: ICertificatesService
 * @Description: 平台证书业务
 * @Author: wanggeng
 * @Date: 2021/8/26 2:40 下午
 * @Version: 1.0
 */
@Service
public class CertificateServiceImpl extends BasePayService implements ICertificateService {

    @Override
    public String getNewestPlatformCertificate() throws Exception {
        String urlSuffix = "/v3/certificates";
        String url = getPayBaseUrl() + urlSuffix;

        LOG.debug("获得证书序列号");
        String serialNumber = getSerialNumber();
        CertificatePayRequestImpl certificatesPayRequest = new CertificatePayRequestImpl();
        String authorization = getAuthorization(RequestMethod.GET, urlSuffix, serialNumber, payProperties, "", PayAuthorizationTypeEnum.WECHATPAY2_SHA256_RSA2048);
        List<Certificate> certificates = certificatesPayRequest.get(url, authorization, serialNumber);
        LOG.debug("获取最新的平台证书");
        Certificate newestCertificate = certificates.get(certificates.size() - 1);
        LOG.debug("解析密文");
        AbstractPayRequest.AesUtil aesUtil = new AbstractPayRequest.AesUtil(payProperties.getApiV3Secretkey().getBytes(StandardCharsets.UTF_8));
        Certificate.EncryptCertificate encryptCertificate = newestCertificate.getEncryptCertificate();
        String ciphertext = aesUtil.decryptToString(encryptCertificate.getAssociatedData().getBytes(StandardCharsets.UTF_8), encryptCertificate.getNonce().getBytes(StandardCharsets.UTF_8), encryptCertificate.getCiphertext());
        String certificateFilePath = payProperties.getCertificatePath() + File.separator + DateUtil.getDays()+ "_wxp_pub.pem";
        LOG.debug("新证书文件路径：{}", certificateFilePath);
        try (FileWriter fileWriter = new FileWriter(certificateFilePath)) {
            fileWriter.write(ciphertext);
            fileWriter.flush();
        } catch (IOException e) {
            LOG.error(e.getMessage(), e);
            throw new SystemException("证书生成异常");
        }
        LOG.debug("保存新平台证书成功");
        return certificateFilePath;
    }
}
