package ink.wgink.register.base.controller.app.api;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.exceptions.DependencyException;
import ink.wgink.exceptions.ParamsException;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.interfaces.sms.ISmsBaseService;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.register.base.pojo.vos.RegisterDefaultVO;
import ink.wgink.register.base.pojo.vos.RegisterPhoneVO;
import ink.wgink.register.base.service.IRegisterService;
import ink.wgink.util.RegexUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiResponse;
import io.swagger.annotations.ApiResponses;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.Map;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: RegisterController
 * @Description: 注册
 * @Author: wanggeng
 * @Date: 2021/4/29 6:12 下午
 * @Version: 1.0
 */
@Api(tags = ISystemConstant.API_TAGS_APP_PREFIX + "注册")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/register")
public class RegisterAppController extends DefaultBaseController {

    @Autowired(required = false)
    private ISmsBaseService smsBaseService;
    @Autowired
    private IRegisterService registerService;

    @ApiOperation(value = "默认注册", notes = "默认注册，用户名密码注册接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("default")
    @CheckRequestBodyAnnotation
    public synchronized SuccessResult defaultRegister(@RequestBody RegisterDefaultVO registerDefaultVO) throws Exception {
        if (!RegexUtil.isUsername(registerDefaultVO.getUsername())) {
            throw new ParamsException("用户名只能包含字母、数字、-_!@#%.");
        }
        if (!StringUtils.equals(registerDefaultVO.getPassword(), registerDefaultVO.getPasswordSame())) {
            throw new ParamsException("两次密码不一致");
        }
        if (!RegexUtil.isPasswordMiddle(registerDefaultVO.getPassword())) {
            throw new ParamsException("密码强度不够, 密码6到16位，包含数字、字母、特殊字符，其中的任意两种");
        }
        Map<String, Object> params = requestParams();
        registerService.registerDefault(registerDefaultVO, params);
        return new SuccessResult();
    }

    @ApiOperation(value = "手机注册", notes = "手机注册，手机 + 短信验证码接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("phone")
    @CheckRequestBodyAnnotation
    public synchronized SuccessResult phone(@RequestBody RegisterPhoneVO registerPhoneVO) throws Exception {
        if (smsBaseService == null) {
            throw new DependencyException("短信依赖未引入");
        }
        String verifyCode = smsBaseService.getVerifyCode(registerPhoneVO.getPhone());
        if (StringUtils.isBlank(verifyCode)) {
            throw new ParamsException("验证码无效");
        }
        if (!StringUtils.equals(verifyCode, registerPhoneVO.getVerifyCode())) {
            throw new ParamsException("验证码错误");
        }
        Map<String, Object> params = requestParams();
        registerService.registerPhone(registerPhoneVO, params);
        return new SuccessResult();
    }

}
