package ink.wgink.service.department.service;

import ink.wgink.interfaces.department.IDepartmentUserBaseService;
import ink.wgink.pojo.dtos.department.DepartmentSimpleDTO;
import ink.wgink.pojo.pos.DepartmentPO;
import ink.wgink.service.department.pojo.vos.DepartmentUserSortVO;

import java.util.List;

/**
 * When you feel like quitting. Think about why you started
 * 当你想要放弃的时候，想想当初你为何开始
 *
 * @ClassName: IDepartmentUserService
 * @Description: 组织部门用户
 * @Author: wanggeng
 * @Date: 2021/1/28 4:21 下午
 * @Version: 1.0
 */
public interface IDepartmentUserService extends IDepartmentUserBaseService {

    /**
     * 用户加入组织组织
     */
    String USER_DEPARTMENT_TYPE_JOIN = "join";
    /**
     * 用户离开组织组织
     */
    String USER_DEPARTMENT_TYPE_LEAVE = "leave";

    /**
     * 新增用户ID
     *
     * @param departmentId      部门ID
     * @param departmentName    部门名称
     * @param departmentUserIds 新增的用户ID列表
     */
    void save(String departmentId, String departmentName, List<String> departmentUserIds);

    /**
     * 删除用户
     *
     * @param departmentId   部门ID
     * @param departmentName 部门名称
     * @param userIds        用户ID列表
     */
    void delete(String departmentId, String departmentName, List<String> userIds);

    /**
     * 删除用户
     *
     * @param departmentId 部门ID
     */
    void delete(String departmentId);

    /**
     * 更新用户
     *
     * @param departmentId 部门ID
     * @param userIds      用户ID
     */
    void update(String departmentId, List<String> userIds);

    /**
     * 更新用户排序
     *
     * @param departmentId
     * @param departmentUserSortVO
     */
    void updateSort(String departmentId, DepartmentUserSortVO departmentUserSortVO);

    /**
     * 部门列表
     *
     * @param userId 用户ID
     * @return
     */
    List<DepartmentPO> listDepartmentPOByUserId(String userId);

    /**
     * 部门列表（简单）
     *
     * @param departmentPOs
     * @return
     */
    List<DepartmentSimpleDTO> listSimple(List<DepartmentPO> departmentPOs);

}
